Strict

Import BRL.Pixmap
Import Pub.LibPNG
Import "factor.bmx"


Private

Global png_stream:TStream

Function png_read_fn( png_ptr,buf:Byte Ptr,size )
	Return png_stream.ReadBytes( buf,size )
End Function

Function png_write_fn( png_ptr,buf:Byte Ptr,size )
	Return png_stream.WriteBytes( buf,size )
End Function

Function png_flush_fn( png_ptr )
	png_stream.Flush
End Function

Public

Function WritePixmapPNG( str:TStream,pixmap:TPixmap,compression=9 )

	compression=Min( Max( compression,0 ),9 )
	
	png_stream=str
	If Not png_stream Return
	
	Try
		Local png_ptr=png_create_write_struct( "1.2.12",Null,Null,Null )
		Local info_ptr=png_create_info_struct( png_ptr )
		
		png_set_write_fn png_ptr,Null,png_write_fn,png_flush_fn
		
		Local bitdepth,colortype,transform
		
		Select pixmap.format
		Case PF_I8
			colortype=PNG_COLOR_TYPE_GRAY
		Case PF_RGB888
			colortype=PNG_COLOR_TYPE_RGB
		Case PF_BGR888
			colortype=PNG_COLOR_TYPE_RGB
			transform=PNG_TRANSFORM_BGR
		Case PF_RGBA8888
			colortype=PNG_COLOR_TYPE_RGBA
		Case PF_BGRA8888
			colortype=PNG_COLOR_TYPE_RGBA
			transform=PNG_TRANSFORM_BGR
		Default
			colortype=PNG_COLOR_TYPE_RGBA
			pixmap=pixmap.Convert( PF_RGBA8888 )
		End Select
	
		png_set_compression_level png_ptr,compression
	
		png_set_IHDR png_ptr,info_ptr,pixmap.width,pixmap.height,8,colortype,PNG_INTERLACE_NONE,PNG_COMPRESSION_TYPE_DEFAULT,PNG_FILTER_TYPE_DEFAULT
		
		Local rows:Byte Ptr[pixmap.height]
		For Local i=0 Until pixmap.height
			rows[i]=pixmap.PixelPtr( 0,i )
		Next
		
		png_set_rows png_ptr,info_ptr,rows
	
		png_write_png png_ptr,info_ptr,transform,Null
	
		png_destroy_write_struct Varptr png_ptr,Varptr info_ptr,Null
		
		'png_stream.Close
		png_stream=Null
		
		Return True
		
	Catch ex:String
	
		If ex<>"PNG ERROR" Throw ex
	
	End Try
		
End Function

Function ReadPixmapPNG:TPixmap( str:TStream )

	png_stream=str
	If Not png_stream Return

	Local buf:Byte[8]
	
	If png_stream.ReadBytes( buf,8 )<>8 Or png_sig_cmp( buf,0,8 )<>0
		'png_stream.Close
		png_stream=Null
		Return
	EndIf
	
	Try
		Local png_ptr=png_create_read_struct( "1.2.12",Null,Null,Null )
		
		' check for valid png_ptr
		If Not png_ptr Then
			'png_stream.Close
			png_stream=Null
			Return
		End If
		
		Local info_ptr=png_create_info_struct( png_ptr )
		
		' check for valid info_ptr
		If Not info_ptr Then
			'png_stream.Close
			png_stream=Null
			png_destroy_read_struct Varptr png_ptr,Null,Null
			Return
		End If
		
		png_set_sig_bytes png_ptr,8
	
		png_set_read_fn png_ptr,Null,png_read_fn
			
		png_read_png png_ptr,info_ptr,PNG_TRANSFORM_EXPAND|PNG_TRANSFORM_STRIP_16,0
	
		Local width,height,bit_depth,color_type,interlace_type,compression_type,filter_method
		png_get_IHDR png_ptr,info_ptr,width,height,bit_depth,color_type,interlace_type,compression_type,filter_method
					
		Local pixmap:TPixmap
				
		If bit_depth=8
			Local pf
			Select color_type
			Case PNG_COLOR_TYPE_GRAY
				pf=PF_I8
			Case PNG_COLOR_TYPE_RGB
				pf=PF_RGB888
			Case PNG_COLOR_TYPE_RGBA
				pf=PF_RGBA8888
			Case PNG_COLOR_TYPE_GRAY_ALPHA
				pixmap=CreatePixmap( width,height,PF_RGBA8888 )
				Local rows:Byte Ptr Ptr=png_get_rows( png_ptr,info_ptr )
				For Local y=0 Until height
					Local src:Byte Ptr=rows[y]
					Local dst:Byte Ptr=pixmap.PixelPtr(0,y)
					For Local x=0 Until width
						Local a=src[0]
						Local i=src[1]
						dst[0]=i
						dst[1]=i
						dst[2]=i
						dst[3]=a
						src:+2
						dst:+4
					Next
				Next
			End Select
			If pf
				pixmap=CreatePixmap( width,height,pf )
				Local rows2:Byte Ptr Ptr=png_get_rows( png_ptr,info_ptr )
				For Local y2=0 Until height
					CopyPixels rows2[y2],pixmap.PixelPtr(0,y2),pf,width
				Next
			EndIf
		EndIf
			
		png_destroy_read_struct Varptr png_ptr,Varptr info_ptr,Null
	
		'png_stream.Close
		png_stream=Null
		Return pixmap
		
	Catch ex:String
	
		If ex<>"PNG ERROR" Throw ex
	
	End Try

End Function

Function ReadIntArray%[](f:TStream)
	Local p:TPixmap=ReadPixmapPNG(f)
	Local a%[PixmapWidth(p)*PixmapHeight(p)]
	For Local x%=0 To PixmapWidth(p)-1
	For Local y%=0 To PixmapHeight(p)-1
		a[x+y*PixmapWidth(p)]=~ReadPixel(p,x,y)
	Next
	Next
	Return a
End Function
Function WriteIntArray(f:TStream,a%[])
	Local size%%[]=squarefactorpair(a.length)
	Local p:TPixmap=CreatePixmap(size[0],size[1],PF_RGBA8888)
	For Local x%=0 To a.length-1
		WritePixel p,x Mod size[0],x/size[1],~a[x]
	Next
	WritePixmapPNG f,p
End Function





