superstrict

import "lib/pixmapgraphics.bmx"
import "lib/input.bmx"
import "lib/rectsoverlap.bmx"
import "lib/randfrom.bmx"
import "lib/fps.bmx"
import "lib/font.bmx"
import "lib/vect2d.bmx"

SetGraphicsDriver GLMax2DDriver()
apptitle = "FURNITURE GHOST"
seedrnd(millisecs())

const GRAPHICSSCALE% = 2
const GRAPHICSW% = 400
const GRAPHICSH% = 300

graphics(GRAPHICSW, GRAPHICSH)
setblend(alphablend)
setclscolor(46, 47, 43)
setmaskcolor(255, 0, 255)

global fon:font = loadfont("graphics/mycelium.ift")

function getargb%(r%, g%, b%, a% = 255)
    return (a shl 24) | (r shl 16) | (g shl 8) | b
end function

function maskpixmap(pix:tpixmap, mask% = $ffff00ff)
    for local maskx% = 0 until pix.width
        for local masky% = 0 until pix.height
            if pix.readpixel(maskx, masky) = mask then
                pix.writepixel(maskx, masky, 0)
            endif
        next
    next
end function

type animpix
    field frames:timage[]
    field width%, height%
    function load:animpix(src$, width%, height%, scale% = 1, mask% = $ffff00ff)
        local pix:tpixmap = loadpixmap(src)
        if width < 0 then width = pix.width / -width
        if height < 0 then height = pix.height / -height
        return animpix.create(pix, width, height, scale, mask)
    end function
    function create:animpix(src:tpixmap, width%, height%, scale% = 1, mask% = $ffff00ff)
        maskpixmap(src, mask)
        local pix:animpix = new animpix
        local framex% = 0
        local framey% = 0
        local frameswide% = (src.width / width)
        local frameshigh% = (src.height / height)
        local framecount% = frameswide * frameshigh
        pix.width = width * scale
        pix.height = height * scale
        pix.frames = new timage[framecount]
        local frameindex% = 0
        while framey < (frameshigh * height)
            while framex < (frameswide * width)
                local thisframe:tpixmap = createpixmap( ..
                    width * scale, height * scale, src.format ..
                )
                thisframe.clearpixels(0)
                PasteSubPixmapRectNN( ..
                    src, thisframe, 0, 0, thisframe.width, thisframe.height, ..
                    framex, framey, width, height ..
                )
                pix.frames[frameindex] = loadimage(thisframe)
                framex :+ width
                frameindex :+ 1
            wend
            framex = 0
            framey :+ height
        wend
        return pix
    end function
    method draw(x%, y%, frame%)
        drawimage(self.frames[frame], x, y)
    end method
end type

function cmpentityz%(o1:object, o2:object)
    local e1:entity = entity(o1)
    local e2:entity = entity(o2)
    if e1.y > e2.y return 1
    if e1.y < e2.y return -1
    return 0
end function
type entity
    global entities:tlist = createlist()
    function sortentities()
        sortlist(entities, true, cmpentityz)
    end function
    function cleanentities()
        local newlist:tlist = createlist()
        for local ent:entity = eachin entities
            if ent.active then newlist.addlast(ent)
        next
        entities = newlist
    end function
    
    field x!, y!, w!, h!
    field collision% = true
    field active% = true
    
    method draw()
    end method
    method update()
    end method
    
    method new()
        entities.addlast(self)
    end method
    method move!(x!, y!)
        if self.collision then
            if x <> 0 and y <> 0 then
                local sum! = 0
                sum :+ self.move(x, 0)
                sum :+ self.move(0, y)
                return sum
            elseif x > 0.75 or y > 0.75 then
                local sum! = 0
                local first! = self.move(x / 2, y / 2)
                if first > 0 then
                    sum :+ first + self.move(x / 2, y / 2)
                endif
                return sum
            else
                if self.y + y < farwally then return 0
                if self.y + y > nearwally then return 0
                if player(self) and self.x + x < viewx - GRAPHICSW * 7/16 then return 0
                for local other:entity = eachin entity.entities
                    if other <> self and other.collision then
                        if rectsoverlap( ..
                            self.x + x, self.y + y - self.h, self.w, self.h, ..
                            other.x, other.y - other.h, other.w, other.h ..
                        ) then return 0
                    endif
                next
                self.x :+ x
                self.y :+ y
                return x + y
            endif
        else
            self.x :+ x
            self.y :+ y
        endif
    end method
end type

type player extends entity
    global KEY_EAST:Key = createkey(KEY_RIGHT)
    global KEY_NORTH:Key = createkey(KEY_UP)
    global KEY_WEST:Key = createkey(KEY_LEFT)
    global KEY_SOUTH:Key = createkey(KEY_DOWN)
    global KEY_PISS:Key = createkey(KEY_Z)
    
    global playerscale% = GRAPHICSSCALE
    global pix:animpix = animpix.load("graphics/player.png", 13, 18, player.playerscale, getargb(156, 175, 190))
    global pissanim:animpix = animpix.load("graphics/piss.png", 4, 6, player.playerscale)
    global pissbarwidth% = 9 * GRAPHICSSCALE

    field walkframe% = -1
    field direction% = 0 ' EAST NORTH WEST SOUTH
    field pissing% = false
    field pisstry% = false
    field pissframe% = 0
    field pissmeter! = 1.0
    field canpiss% = false
    
    field nocontrol% = false
    
    field hp% = 4
    field hurtflash% = -1
    field dead% = false
    field score% = 0
    
    method new()
        self.w = 5 * player.playerscale
        self.h = 3 * player.playerscale
    end method
    
    method hurt()
        self.hp :- 1
        self.hurtflash = 120
        if self.hp <= 0 then
            self.hp = 0
            self.dead = true
        endif
    end method
    
    method draw()
        local frame% = self.direction * 5
        if self.pisstry then
            frame :+ 1
        elseif self.walkframe >= 0 then
            local walking% = (self.walkframe / 8) mod 4
            if walking >= 3 then walking = 1
            frame :+ walking + 2
        endif
        attemptdrawpiss(true)
        setalpha(0.25)
        setcolor(0, 0, 0)
        drawrect(self.x - 3*playerscale, self.y - self.h/2, self.w + 6*playerscale, self.h)
        setalpha(1)
        local hurtframe% = (self.hurtflash / 6) mod 2
        if (not self.dead) and (self.hurtflash < 0 or hurtframe = 0) then
            setcolor(255, 255, 255)
        else
            setcolor(255, 200, 200)
        endif
        pix.draw(int(self.x) - 4 * playerscale, int(self.y) - pix.height, frame)
        attemptdrawpiss(false)
    end method
    method attemptdrawpiss(bg%)
        if self.pisstry then
            if bg and self.direction = 1 then drawpiss()
            if (not bg) and self.direction <> 1 then drawpiss()
        endif
    end method
    method drawpiss()
        local frame% = (pissframe / 5) mod 2
        local offx% = 0, offy% = 0
        if self.direction = 0 then
            offx :+ 7
        elseif self.direction = 1 then
            offx :+ 2
            offy :- 1
        elseif self.direction = 2 then
            offx :- 6
        elseif self.direction = 3 then
            offx :+ 2
            offy :+ 1
        endif
        if self.pissing then
            pissanim.draw(self.x + offx*GRAPHICSSCALE, self.y + offy*GRAPHICSSCALE - pissanim.height, frame + self.direction*2)
            pissframe :+ 1
        endif
        setcolor(80, 70, 0)
        drawrect(self.x - 2*GRAPHICSSCALE, self.y - pix.height - 3*GRAPHICSSCALE, pissbarwidth, GRAPHICSSCALE)
        setcolor(255, 250, 10)
        drawrect(self.x - 2*GRAPHICSSCALE, self.y - pix.height - 3*GRAPHICSSCALE, pissbarwidth * self.pissmeter, GRAPHICSSCALE)
        setcolor(255, 255, 255)
    end method
    
    method update()
        if self.hurtflash >= 0 then self.hurtflash :- 1
        if self.nocontrol then return
        self.pisstry = player.KEY_PISS.down() and self.canpiss
        self.pissing = self.pisstry and self.pissmeter > 0
        if player.KEY_EAST.hit() then
            self.direction = 0
        elseif player.KEY_NORTH.hit() then
            self.direction = 1
        elseif player.KEY_WEST.hit() then
            self.direction = 2
        elseif player.KEY_SOUTH.hit() then
            self.direction = 3
        endif
        if player.KEY_EAST.released() or player.KEY_WEST.released() then
            if player.KEY_NORTH.down() then
                self.direction = 1
            elseif player.KEY_SOUTH.down() then
                self.direction = 3
            endif
        endif
        if player.KEY_NORTH.released() or player.KEY_SOUTH.released() then
            if player.KEY_EAST.down() then
                self.direction = 0
            elseif player.KEY_WEST.down() then
                self.direction = 2
            endif
        endif
        if not pisstry then
            self.pissmeter :+ 0.0005
            if self.pissmeter > 1 then self.pissmeter = 1
            
            local movex! = 0
            local movey! = 0
            local movespeed% = 1.0 * GRAPHICSSCALE
            movex :+ player.KEY_EAST.down()
            movey :- player.KEY_NORTH.down()
            movex :- player.KEY_WEST.down()
            movey :+ player.KEY_SOUTH.down()
            if abs(movex) + abs(movey) > 1 then
                movex :/ 1.42
                movey :/ 1.42
            endif
            self.move(movex * movespeed, movey * movespeed)
            if movex = 0 and movey = 0 then
                self.walkframe = -1
            else
                self.walkframe :+ 1
            endif
        else
            self.pissmeter :- 0.005
            if self.pissmeter < 0 then self.pissmeter = 0
            if self.pissing then
                local stainx% = 0, stainy% = 0
                if self.direction = 0 then
                    stainx :+ 10
                elseif self.direction = 1 then
                    stainx :+ 2
                    stainy :- 1
                elseif self.direction = 2 then
                    stainx :- 6
                elseif self.direction = 3 then
                    stainx :+ 2
                    stainy :+ 1
                endif
                stain.add(self.x + stainx*GRAPHICSSCALE, self.y + stainy*GRAPHICSSCALE)
            endif
        endif
    end method
end type

function loadgraphic:timage(path$, scale% = -1, mask% = $ffff00ff)
    if scale = -1 then scale = GRAPHICSSCALE
    local srcpix:tpixmap = loadpixmap(path)
    maskpixmap(srcpix)
    if scale = 1 then
        return loadimage(srcpix)
    else
        local scaledpix:tpixmap = createpixmap(srcpix.width * scale, srcpix.height * scale, srcpix.format)
        scaledpix.clearpixels(0)
        PasteSubPixmapRectNN(srcpix, scaledpix, 0, 0, scaledpix.width, scaledpix.height, 0, 0, srcpix.width, srcpix.height)
        return loadimage(scaledpix)
    endif
end function

type propdata
    const IMGDIM% = -1
    global propscale% = GRAPHICSSCALE
    global props:tlist = createlist()
    
    global fridge:propdata = propdata.create("fridge", IMGDIM, 5)
    global oven:propdata = propdata.create("oven", IMGDIM, 8)
    global sofa_south:propdata = propdata.create("sofa_south", IMGDIM, 8)
    global coffeetable:propdata = propdata.create("coffeetable", IMGDIM, 9)
    global dinnertable:propdata = propdata.create("dinnertable", IMGDIM, 8, 0.6)
    global bed:propdata = propdata.create("bed", IMGDIM, 22, 0.35)
    global laptop:propdata = propdata.create("laptop", IMGDIM, 5, 0.1, false) ' no ghost alt
    global grandfatherclock:propdata = propdata.create("grandfatherclock", IMGDIM, 9, 0.1, false) ' no ghost alt
    global crate:propdata = propdata.create("crate", IMGDIM, 7, 0.3)
    global filecabinet:propdata = propdata.create("filecabinet", IMGDIM, 6, 0.8)
    global dresser:propdata = propdata.create("dresser", IMGDIM, 8)
    
    global oildrumgreen:propdata = propdata.create("oildrumgreen", IMGDIM, 5, 0.2)
    global oildrumbrown:propdata = propdata.create("oildrumbrown", IMGDIM, 5, 0.2)
    
    global toilet:propdata = propdata.create("toilet", IMGDIM, 8, 0.3)
    global sink:propdata = propdata.create("sink", IMGDIM, 5, 0.3)
    
    global emptybookshelf:propdata = propdata.create("emptybookshelf", IMGDIM, 7, 0.4)
    global fullbookshelf:propdata = propdata.create("fullbookshelf", IMGDIM, 7, 0.4)
    
    global chair_east:propdata = propdata.create("chair_east", IMGDIM, 5, 0.2)
    global chair_north:propdata = propdata.create("chair_north", IMGDIM, 5, 0.2, false) ' no ghost alt
    global chair_west:propdata = propdata.create("chair_west", IMGDIM, 5, 0.2)
    global chair_south:propdata = propdata.create("chair_south", IMGDIM, 5, 0.2)
    
    function getrand:propdata(mustghost% = false)
        local bestprop:propdata = null
        ' Super quick stupid hackish way to weight selections
        local i% = 0
        while i < 8 or bestprop = null
            i :+ 1
            local option:propdata = propdata(RandFromList(propdata.props))
            if mustghost and not option.ghostallowed then continue
            if bestprop = null or rnd(option.weight) >= rnd(bestprop.weight) then bestprop = option
        wend
        return bestprop
    end function
        
    field image:animpix
    field w!, h!
    field ghostallowed% = true
    field weight! = 1
    
    function create:propdata(name$, width% = IMGDIM, height% = IMGDIM, weight! = 1, ghostallowed% = true)
        local prop:propdata = new propdata
        prop.weight = weight
        prop.ghostallowed = ghostallowed
        prop.image = animpix.load("graphics/props/" + name + ".png", -2, -1, propscale)
        prop.w = width * propscale
        prop.h = height * propscale
        if width < 0 then prop.w = prop.image.width
        if height < 0 then prop.h = prop.image.height
        props.addlast(prop)
        return prop
    end function
    method draw(x%, y%, frame% = 0)
        setalpha(0.25)
        setcolor(0, 0, 0)
        drawrect(x, y - self.h + 2 * propdata.propscale, self.w, self.h)
        setalpha(1)
        setcolor(255, 255, 255)
        self.image.draw(x, y - self.image.height, frame)
    end method
end type
type prop extends entity
    field data:propdata
    function create:prop(data:propdata, x!, y!)
        local pro:prop = new prop
        pro.data = data
        pro.x = x
        pro.y = y
        pro.w = data.w
        pro.h = data.h
        return pro
    end function
    method draw()
        self.data.draw(self.x, self.y)
    end method
end type

type ghost extends entity
    global graphic:timage = loadgraphic("graphics/ghost.png")
    
    field runahead% = false
    field rungoal% = 0
    field runningy! = 0
    field hiddenas:propdata = null
    field pissed% = false
    field chasing% = false
    field runaheadwait% = -1
    
    method new()
        self.collision = false
        self.w = 0
        self.h = 0
    end method
    method draw()
        if self.runahead or self.chasing then
            setalpha(rnd(0.1, 1))
            if self.pissed then setcolor(255, 255, rand(110, 250))
            drawimage(graphic, self.x + rnd(-1, 1), self.y - graphic.height + rnd(-1, 1))
            setalpha(1)
            setcolor(255, 255, 255)
        elseif self.hiddenas <> null
            self.hiddenas.draw(self.x, self.y, 1)
        endif
    end method
    method chase()
        local targetx% = play.x - 4*GRAPHICSSCALE
        local targety% = play.y - 7*GRAPHICSSCALE
        local vec:vect2d = vect2d.create(targetx - self.x, targety - self.y)
        vec.applynormalize()
        vec.applyscale(4.25)
        self.x :+ vec.x
        self.y :+ vec.y
        if rectsoverlap(self.x, self.y, 1, 1, targetx-2, targety-2, 5, 5) then
            play.hurt()
            self.chasing = false
            self.startrunahead()
            self.runaheadwait = 40
        endif
    end method
    method startrunahead()
        self.runahead = true
        self.runningy = max(0.15, rnd())
        if rand(0, 1) = 1 then self.runningy :* -1
        self.rungoal = GRAPHICSW * rnd(1.0, 2.70)
    end method
    method stoprunahead()
        self.runahead = false
        self.hide()
        for local ent:entity = eachin entity.entities
            if prop(ent) and rectsoverlap(ent.x, ent.y, ent.w, ent.h, self.x, self.y, self.w, self.h) then
                ent.active = false
            endif
        next
    end method
    method hide()
        self.hiddenas = propdata.getrand(true)
        self.w = self.hiddenas.w
        self.h = self.hiddenas.h
        self.pissed = false
        refreshprops()
    end method
    method update()
        self.collision = not self.runahead
        if self.chasing then
            self.chase()
        elseif self.runahead
            if self.x - viewx > self.rungoal then
                self.stoprunahead()
            elseif self.runaheadwait <= 0
                self.x :+ 4
                self.y :+ self.runningy
                if self.runningy < 0 and self.y <= farwally + 10*GRAPHICSSCALE then
                    self.runningy :* -1.2
                elseif self.runningy > 0 and self.y >= nearwally - 10*GRAPHICSSCALE then
                    self.runningy :* -1.2
                endif
            else
                runaheadwait :- 1
            endif
        elseif viewx - self.x > (GRAPHICSW * 3 / 4) then
            self.chasing = true
        elseif self.hiddenas <> null
            for local st:stain = eachin stain.stains
                local renderw% = st.renderw()
                local renderh% = st.renderw()
                if st.size >= 6 and rectsoverlap(st.x - renderw/2, st.y - renderh/2, renderw, renderh, self.x, self.y, self.w, self.h) then
                    play.score :+ 1
                    self.pissed = true
                    self.startrunahead()
                endif
            next
        endif
    end method
    
end type

type stain ' piss stain
    global stains:tlist = createlist()
    
    function drawall()
        for local st:stain = eachin stains
            st.draw()
        next
    end function
    function updateall()
        local newlist:tlist = null
        if stains.count() > 32 then newlist = createlist()
        for local st:stain = eachin stains
            if st.opacity > 0.6 then st.opacity = 0.6
            st.opacity :- 0.001
            if newlist <> null and st.opacity > 0 and st.x >= viewx - GRAPHICSW/2 - st.size*3 then
                newlist.addlast(st)
            endif
        next
        if newlist then stains = newlist
    end function
    function add:stain(x!, y!)
        for local st:stain = eachin stains
            local renderw% = st.renderw()
            local renderh% = st.renderh()
            if rectsoverlap(st.x - renderw/2, st.y - renderh/2, renderw, renderh, x-1, y-1, 3, 3) then
                st.size :+ 0.125
                st.opacity :+ 0.0015
                return st
            endif 
        next
        local st:stain = new stain
        st.x = x
        st.y = y
        return st
    end function
    
    field x!, y!
    field size! = 2
    field opacity! = 0.6
    
    method new()
        stains.addlast(self)
    end method
    
    method renderw%()
        return int(min(self.size, 12)) * GRAPHICSSCALE
    end method
    method renderh%()
        return int(min(self.size, 12)) * GRAPHICSSCALE/3
    end method
    
    method draw()
        if self.opacity <= 0 then return
        local renderw% = self.renderw()
        local renderh% = self.renderh()
        setalpha(self.opacity)
        setcolor(30, 20, 0)
        drawrect(self.x - renderw/2, self.y - renderh/2, renderw, renderh)
        setalpha(1)
        setcolor(255, 255, 255)
    end method
end type

function refreshprops()
    for local ent:entity = eachin entity.entities
        if ent.x < viewx - GRAPHICSW / 2 - ent.w * 2 - 10 then
            ent.active = false
        endif
    next
    entity.cleanentities()
    local addgoal% = rand(50, 70)
    local added% = 0, failed% = 0
    while added < addgoal and failed < 180
        local data:propdata = propdata.getrand()
        local addx% = rand(viewx + GRAPHICSW * 1.10, viewx + GRAPHICSW * 4.5)
        local addy% = rand(farwally + 2 + data.h, nearwally - 10)
        local canadd% = true
        for local ent:entity = eachin entity.entities
            if not prop(ent) or ghost(ent) then continue
            local dx% = ent.x - addx
            local dy% = ent.y - addy
            if abs(dx) + abs(dy) < max(ent.w, ent.h) + max(data.w, data.h) + GRAPHICSSCALE then
                canadd = false
                exit
            endif
        next
        if canadd
            entity.entities.addlast(prop.create(data, addx, addy))
            added :+ 1
        else
            failed :+  1
        endif
    wend
end function

function updateview(play:player)
    local xthresh% = GRAPHICSW / 4
    local ythresh% = GRAPHICSH / 4
    
    local px% = play.x - play.w / 2
    local py% = play.y - play.h / 2
    
    if viewx - px > xthresh then
        'viewx = px + xthresh
    elseif viewx - px < -xthresh then
        viewx = px - xthresh
    endif
    if viewy - py > ythresh then
        viewy = py + ythresh
    elseif viewy - py < -ythresh then
        viewy = py - ythresh
    endif
end function

global wall_front:timage = loadgraphic("graphics/wall_front.png")
global wall_back:timage = loadgraphic("graphics/wall_back.png")
global wallimagewidth% = min(wall_front.width, wall_back.width)

global nearwally% = 200 * GRAPHICSSCALE
global farwally% = 0

function drawnearwall()
    setcolor(0, 0, 0)
    drawrect(viewx - GRAPHICSW / 2, nearwally + wall_back.height, GRAPHICSW, GRAPHICSH)
    setcolor(255, 255, 255)
    drawwallimage(wall_back, nearwally)
end function
function drawfarwall()
    local wally% = farwally - wall_front.height
    setcolor(0, 0, 0)
    drawrect(viewx - GRAPHICSW / 2, -GRAPHICSH + wally, GRAPHICSW, GRAPHICSH)
    setcolor(255, 255, 255)
    drawwallimage(wall_front, wally)
end function
function drawwallimage(image:timage, drawy%)
    local startx% = (viewx - GRAPHICSW / 2 - wallimagewidth) / wallimagewidth * wallimagewidth - 10
    local finishx% = (viewx + GRAPHICSW / 2 + wallimagewidth) / wallimagewidth * wallimagewidth
    local drawx% = startx
    while drawx < finishx
        drawimage(image, drawx, drawy)
        drawx :+ image.width
    wend
end function

global hudimage:animpix = animpix.load("graphics/hud.png", -2, -2, GRAPHICSSCALE, $ff000000)
function drawhud()
    local originx#, originy#
    getorigin(originx, originy)
    setorigin(0, 0)
    local stuffx% = 10*GRAPHICSSCALE
    local heartsy% = 10*GRAPHICSSCALE
    local starsy% = 19*GRAPHICSSCALE
    local xinc% = 8*GRAPHICSSCALE
    setalpha(0.25)
    setcolor(0, 0, 0)
    drawrect(stuffx - GRAPHICSSCALE, heartsy - GRAPHICSSCALE, xinc*4 + GRAPHICSSCALE*2, 17*GRAPHICSSCALE)
    setalpha(1)
    setcolor(255, 255, 255)
    for local i% = 0 until 4
        local heartframe% = 0
        local starframe% = 0
        if i >= play.hp then heartframe = 1
        if i >= play.score then starframe = 1
        hudimage.draw(stuffx, heartsy, heartframe)
        hudimage.draw(stuffx, starsy, starframe + 2)
        stuffx :+ xinc
    next
    setorigin(originx, originy)
end function

global viewx% = 0
global viewy% = 0

global halfy% = (nearwally + farwally)/2

global play:player = null
global gho:ghost = null

global fps:framelimiter = framelimiter.create(60)

const GAME_PLAY% = 0
const GAME_WON% = 1
const GAME_LOST% = 2
global gamestate% = 0
global fadein! = 1

intro()
initgame()
mainloop()

function initgame()
    gamestate = GAME_PLAY
    entity.entities = createlist()
    viewx = 0
    viewy = 0
    play = new player
    play.x = 0
    play.y = halfy
    play.canpiss = false
    play.hp = 4
    play.score = 0
    gho = new ghost
    gho.x = -50
    gho.y = halfy + rand(-25, 25)*GRAPHICSSCALE
    prop.create(propdata.toilet, -30*GRAPHICSSCALE, halfy)
    prop.create(propdata.sink, -20*GRAPHICSSCALE, halfy)
    fadein = 1
    updateview(play)
end function
function mainloop()
    setclscolor(46, 47, 43)
    play.canpiss = false
    local gameoverframe% = 0
    local gameovery! = 0
    
    flushkeys()
    repeat
        
        if not player.KEY_PISS.down() then play.canpiss = true
        
        setorigin(-viewx + GRAPHICSW / 2, -viewy + GRAPHICSH / 2)
        
        if gamestate = GAME_PLAY then
            play.update()
            for local ent:entity = eachin entity.entities
                if ent <> play then ent.update()
            next
            stain.updateall()
            updateview(play)
            
            if play.hp <= 0 then
                gamestate = GAME_LOST
                flushkeys()
            elseif play.score >= 4 then
                gamestate = GAME_WON
                flushkeys()
            endif
        else
            gho.update()
            gameoverframe :+ 1
            gameovery = sin(gameoverframe / 2) * 0.2
        endif
        
        stain.drawall()
        drawfarwall()
        
        entity.sortentities()
        for local ent:entity = eachin entity.entities
            ent.draw()
        next
        
        drawnearwall()
        
        drawhud()
        
        if fadein > 0 then
            local originx#, originy#
            setalpha(fadein)
            setcolor(0, 0, 0)
            drawrect(viewx -GRAPHICSW/2 - 10, viewy -GRAPHICSH/2 - 10, GRAPHICSW + 20, GRAPHICSH + 20)
            setalpha(1)
            setcolor(255, 255, 255)
            fadein :- 0.005
        endif
        
        if gamestate = GAME_WON then
            text(0.45 + gameovery, "YOU WIN")
        elseif gamestate = GAME_LOST then
            text(0.45 + gameovery, "GAME OVER")
        endif
        
        if gamestate <> GAME_PLAY and player.KEY_PISS.hit() then initgame()
        
        flip -1
        fps.update()
        cls
        if keydown(KEY_ESCAPE) or appterminate() then end
        
    forever
end function

function text(y!, str$)
    local originx#, originy#
    getorigin(originx, originy)
    setorigin(0, 0)
    setcolor(0, 0, 0)
    setscale(GRAPHICSSCALE, GRAPHICSSCALE)
    fon.draw(str, GRAPHICSW/2, y*GRAPHICSH + GRAPHICSSCALE, font.center)
    setcolor(255, 255, 255)
    fon.draw(str, GRAPHICSW/2, y*GRAPHICSH, font.center)
    setorigin(originx, originy)
    setscale(1, 1)
end function
function intro()
    global introanim:animpix = animpix.load("graphics/intropc.png", -2, -1, GRAPHICSSCALE, $ffffffff)
    local introframe% = 0
    local clsr0% = 36, clsg0% = 37, clsb0% = 33
    local clsr1% = 80, clsg1% = 10, clsb1% = 12
    local clsstate! = 0
    local clsdir% = 1
    local ghostx! = GRAPHICSW/2
    local ghostxdir% = 1
    
    flushkeys()
    repeat
        
        introframe :+ 1
        local pcanimframe% = (introframe / 10) mod 2
        
        introanim.draw((GRAPHICSW - introanim.width)/2, (GRAPHICSH - introanim.height)/2, pcanimframe)
        
        text(0.04, "FURNITURE GHOST")
        text(0.14, "BY SOPHIE KIRSCHNER FOR LD35")
        text(0.8, "ARROWS TO MOVE Z TO PISS")
        text(0.9, "PRESS Z TO BEGIN")
        
        ghostx :+ GRAPHICSSCALE * ghostxdir
        if ghostx < GRAPHICSW/8 then ghostxdir = 1
        if ghostx >= GRAPHICSW * 7/8 then ghostxdir = -1
        local ghosty! = GRAPHICSH/2 * sin(introframe) + GRAPHICSH/2
        'setalpha(rnd(0.1, 1))
        drawimage(ghost.graphic, ghostx - ghost.graphic.width/2, ghosty - ghost.graphic.height/2)
        setalpha(1)
        
        flip -1
        fps.update()
        
        setclscolor( ..
            clsr0 * (1-clsstate) + clsr1 * clsstate, ..
            clsg0 * (1-clsstate) + clsg1 * clsstate, ..
            clsb0 * (1-clsstate) + clsb1 * clsstate ..
        )
        clsstate :+ 0.005 * clsdir
        if clsstate < 0 then clsstate = 0; clsdir = 1
        if clsstate > 1 then clsstate = 1; clsdir = -1
        
        cls
        if keydown(KEY_ESCAPE) or appterminate() then end
        if keyhit(KEY_Z) then exit
        
    forever
end function


